@@ -42,12 +42,9 @@ from typing import (
     Dict,
     Iterator,
     List,
-    Match,
     Optional,
     Set,
-    Union,
     Tuple,
-    Type,
     Sequence,
 )
 from . import inspection
@@ -197,28 +194,28 @@ def few_enough_underscores(current: str, match: str) -> bool:
     return not match.startswith("_")
 
 
-def method_match_none(word: str, size: int, text: str) -> bool:
+def _method_match_none(word: str, size: int, text: str) -> bool:
     return False
 
 
-def method_match_simple(word: str, size: int, text: str) -> bool:
+def _method_match_simple(word: str, size: int, text: str) -> bool:
     return word[:size] == text
 
 
-def method_match_substring(word: str, size: int, text: str) -> bool:
+def _method_match_substring(word: str, size: int, text: str) -> bool:
     return text in word
 
 
-def method_match_fuzzy(word: str, size: int, text: str) -> Optional[Match]:
-    s = r".*%s.*" % ".*".join(list(text))
-    return re.search(s, word)
+def _method_match_fuzzy(word: str, size: int, text: str) -> bool:
+    s = r".*{}.*".format(".*".join(c for c in text))
+    return re.search(s, word) is not None
 
 
-MODES_MAP = {
-    AutocompleteModes.NONE: method_match_none,
-    AutocompleteModes.SIMPLE: method_match_simple,
-    AutocompleteModes.SUBSTRING: method_match_substring,
-    AutocompleteModes.FUZZY: method_match_fuzzy,
+_MODES_MAP = {
+    AutocompleteModes.NONE: _method_match_none,
+    AutocompleteModes.SIMPLE: _method_match_simple,
+    AutocompleteModes.SUBSTRING: _method_match_substring,
+    AutocompleteModes.FUZZY: _method_match_fuzzy,
 }
 
 
@@ -231,7 +228,7 @@ class BaseCompletionType:
         mode: AutocompleteModes = AutocompleteModes.SIMPLE,
     ) -> None:
         self._shown_before_tab = shown_before_tab
-        self.method_match = MODES_MAP[mode]
+        self.method_match = _MODES_MAP[mode]
 
     @abc.abstractmethod
     def matches(
@@ -346,13 +343,14 @@ class ImportCompletion(BaseCompletionType):
         return after_last_dot(word)
 
 
+def _safe_glob(pathname: str) -> Iterator[str]:
+    return glob.iglob(glob.escape(pathname) + "*")
+
+
 class FilenameCompletion(BaseCompletionType):
     def __init__(self, mode: AutocompleteModes = AutocompleteModes.SIMPLE):
         super().__init__(False, mode)
 
-    def safe_glob(self, pathname: str) -> Iterator[str]:
-        return glob.iglob(glob.escape(pathname) + "*")
-
     def matches(
         self, cursor_offset: int, line: str, **kwargs: Any
     ) -> Optional[Set]:
@@ -362,7 +360,7 @@ class FilenameCompletion(BaseCompletionType):
         matches = set()
         username = cs.word.split(os.path.sep, 1)[0]
         user_dir = os.path.expanduser(username)
-        for filename in self.safe_glob(os.path.expanduser(cs.word)):
+        for filename in _safe_glob(os.path.expanduser(cs.word)):
             if os.path.isdir(filename):
                 filename += os.path.sep
             if cs.word.startswith("~"):
@@ -374,7 +372,6 @@ class FilenameCompletion(BaseCompletionType):
         return lineparts.current_string(cursor_offset, line)
 
     def format(self, filename: str) -> str:
-        filename.rstrip(os.sep).rsplit(os.sep)[-1]
         if os.sep in filename[:-1]:
             return filename[filename.rindex(os.sep, 0, -1) + 1 :]
         else:
@@ -480,9 +477,11 @@ class DictKeyCompletion(BaseCompletionType):
         r = self.locate(cursor_offset, line)
         if r is None:
             return None
-        curDictParts = lineparts.current_dict(cursor_offset, line)
-        assert curDictParts, "current_dict when .locate() truthy"
-        _, _, dexpr = curDictParts
+        current_dict_parts = lineparts.current_dict(cursor_offset, line)
+        if current_dict_parts is None:
+            return None
+
+        _, _, dexpr = current_dict_parts
         try:
             obj = safe_eval(dexpr, locals_)
         except EvaluationError:
@@ -571,15 +570,15 @@ class ParameterNameCompletion(BaseCompletionType):
         r = self.locate(cursor_offset, line)
         if r is None:
             return None
-        if argspec:
-            matches = {
-                f"{name}="
-                for name in argspec[1][0]
-                if isinstance(name, str) and name.startswith(r.word)
-            }
-            matches.update(
-                name + "=" for name in argspec[1][4] if name.startswith(r.word)
-            )
+
+        matches = {
+            f"{name}="
+            for name in argspec[1][0]
+            if isinstance(name, str) and name.startswith(r.word)
+        }
+        matches.update(
+            name + "=" for name in argspec[1][4] if name.startswith(r.word)
+        )
         return matches if matches else None
 
     def locate(self, cursor_offset: int, line: str) -> Optional[LinePart]:
